from pymongo import MongoClient
from datetime import datetime
from bson import ObjectId

from modele.contenu_dao import ContenuDao
from modele.encours_dao import EncoursDao


class DemandePubDao:
    client = MongoClient('localhost', 27017)

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    ## Crétion d'une demande de publication
    # @param idCOntenu l'id du contenu d'origine
    # @param auteur l'auteur du contenu d'origine
    # @param idContenuEnCours l'id du contenu en cours
    # @param auteurEnCours l'auteur du contenu en cours

    def creer_demande_publication(self, id_contenu, id_contenu_en_cours, auteur_en_cours, auteur, version_en_cours, source):

        self.base_donnees.demandesPub.insert_one(
            {"date": datetime.now().strftime("%d/%m/%y"), "auteurDemande": auteur_en_cours,
             "auteurContenu": auteur, "idContenu": ObjectId(id_contenu),
             "titre": EncoursDao().get_titre_en_cours(id_contenu_en_cours),
             "idContenuEnCours": ObjectId(id_contenu_en_cours), "version_en_cours": version_en_cours,
             "source": source, "etat": "enCours"})

        self.base_donnees.encours.update_one({"_id": ObjectId(id_contenu_en_cours)}, {"$set": {"isPublie": "oui"}})

    ## Récupérer une demande de publication via son id
    # @param id_demande_pub un id de demande de publication
    # @return la demande de publication désirée
    def get_demande_pub_via_id(self, id_demande_pub):
        return self.base_donnees.demandesPub.find_one({"_id": ObjectId(id_demande_pub)})

    ## Récupérer la dernière insertion dasn les demaindes de publication
    # @return le dernier id inséré dans les demandes de publication
    def id_derniere_demande_publication_ajoutee(self):
        # créer un fichier (au bon endroit)
        # récupérer l'id mongo (pour nommer le répertoire)
        tmp = self.base_donnees.demandesPub.find().sort("_id", -1).limit(1)

        record = next(tmp, None)
        return str(record.get("_id"))

    ## Fonction permettant de gérer une demande de publication
    # @param idPublication l'id de la demande de publication
    # @param reponse la réponse donnée par l'auteur d'origine
    # @param auteur l'auteur d'origine
    def gerer_demande_pub(self, id_publication, reponse, auteur_en_cours):
        if reponse == 'o':
            # branche = self.get_branche_contenu_en_cours(id_en_cours)

            # on récupère l'id du contenu d'origine
            id_contenu = self.get_id_contenu_associe_a_une_publication(id_publication)

            # et on met à jour le contenu (sans en modifier l'auteur)
            ContenuDao().maj_contenu(id_publication, None, False)

            # on notifie au 'demandeur' que sa demande de publication est acceptée
            self.base_donnees.demandesPub.update_one({"_id": ObjectId(id_publication)}, {"$set": {"etat": "accepté"}})

            # On met à jour le tiers (chez le 'demandeur' de la publication)
            version = ContenuDao().get_version_courante(id_contenu)
            ContenuDao().maj_tiers(id_contenu, version, auteur_en_cours)

            self.rendre_publications_obsoletes(id_contenu, auteur_en_cours)

        else:
            # on change l'état de la demande de publication
            self.base_donnees.demandesPub.update_one({"_id": ObjectId(id_publication)}, {"$set": {"etat": "refus"}})

    ## Quand unhe publication est validée, toutes les autres publications en cours sur le contenu passent en statut "obsolete"
    # @param id_contenu l'id du contenu lié à la publication acceptée
    def rendre_publications_obsoletes(self, id_contenu, auteur_en_cours):
        # 1. On récupère toutes les publications liées à l'id id_contenu
        all_publications = self.base_donnees.demandesPub.find({"$and": [{"idContenu": ObjectId(id_contenu)},
                                                                        {"etat": "enCours"},
                                                                        {"auteurDemande": {"$ne": auteur_en_cours}}
                                                                        ]})

        for all_entries in all_publications:
            self.base_donnees.demandesPub.update({"_id": all_entries["_id"]}, {"$set": {"etat": "obsolete"}})

    ## Supprimer une demande de publication
    # @param idPub l'id de la demande de publication
    def supprimer_publication(self, idPub):
        self.base_donnees.demandesPub.delete_one({"_id": ObjectId(idPub)})

    ## Récupérer l'auteur d'une demande de publication
    # @param idPub l'id de la demande de publication
    # @return l'auteur de la publication
    def get_auteur_demande_publication(self, idDdePublication):
        return self.base_donnees.demandesPub.find_one({"_id": ObjectId(idDdePublication)})["auteurDemande"]

    ## Récupérer lid du contenu (perso ou tiers) associé à une demande de publication
    # @param idDdePublication l'id de la demande de publication
    # @return l'id du contenu associé à la demande de publication
    def get_id_contenu_associe_a_une_publication(self, id_dde_publication):
        if type(id_dde_publication) == str:
            id_dde_publication = ObjectId(id_dde_publication)

        return self.base_donnees.demandesPub.find_one({"_id": id_dde_publication})["idContenu"]

    ## Récupérer lid du contenu en cours associé à une demande de publication
    # @param idDdePublication l'id de la demande de publication
    # @return l'id du contenu en cours associé à la demande de publication
    def get_id_contenu_en_cours_associe_a_une_publication(self, id_dde_publication):
        if type(id_dde_publication) == str:
            id_dde_publication = ObjectId(id_dde_publication)

        return self.base_donnees.demandesPub.find_one({"_id": id_dde_publication})["idContenuEnCours"]

    ## Récupérer les demandes de publication
    # @param auteur l'auteur des demandes
    # @param emises si true, récupère les demandes emises; les demandes recues sinon
    # @return les demandes de publications idoines
    def get_demandes_pub(self, auteur, emises=True):
        if emises:
            return self.base_donnees.demandesPub.find({"auteurDemande": auteur})
        else:
            return self.base_donnees.demandesPub.find({"auteurContenu": auteur})

    ## récupérer le dernier id inséré dans encours
    # @return l'id de la derniere insertion dans encours.
    def recuperer_dernier_id(self):
        dernier_id_insere = self.base_donnees.demandesPub.find().sort("_id", -1).limit(1)

        record = next(dernier_id_insere, None)
        return str(record.get("_id"))
