
from testUtils import *


def supprimer_les_contenus(client, id, id_en_cours):
    # suppression du contenu en_cours

    rv = client.delete("/en_cours/supprimerEnCoursParId/" + json.dumps(id_en_cours), headers=get_headers(client))
    assert rv.status_code == 200

    # suppression du contenu "intial"
    rv = client.delete("/contenus/deletePerso/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200


def test_en_cours(client):
    # on récupère l'id du contenu en cours créé
    # Dans un premier temps, créer un contenu
    login = creerUtilisateurRandom(client)
    id = creerContenuRandom(client, login)


    data = {"auteur": login, "id_contenu": json.dumps(id), "provenance": "perso", "version": 1}
    # création d'un contenu en_cours
    rv = client.post("/en_cours/newEnCours", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    id_en_cours = json.loads(rv.data)["_id"]

    # récupération du texte (avant modif)
    rv = client.get("/en_cours/getTexteContenuEnCours/" + json.dumps(id_en_cours), headers=get_headers(client))
    assert rv.status_code == 200
    assert "Texte du contenu" == json.loads(rv.data)

    # modification du texte du contenu en cours
    data = {"nouveau_texte": "Nouveau Texte du contenu", "nouveau_titre": "nouveauTitreContenu"}
    rv = client.patch("/contenus/changerTexte/" + json.dumps(id_en_cours), json=data, headers=get_headers(client))
    assert rv.status_code == 200

    # récupération du texte (après modif)
    rv = client.get("/en_cours/getTexteContenuEnCours/" + json.dumps(id_en_cours), headers=get_headers(client))
    assert rv.status_code == 200
    assert "Nouveau Texte du contenu" == json.loads(rv.data)

    supprimer_les_contenus(client, id, id_en_cours)
    supprimerUtilisateur(client, login)


def test_en_cours_provenance_tiers(client):

    login = creerUtilisateurRandom(client)
    id = creerContenuRandom(client, login)


    loginDeux = creerUtilisateurRandom(client)
    # création d'un tiers
    data = {"id_tiers": json.dumps(id), "auteur": loginDeux, "version": 1}
    rv = client.patch("/contenus/ajouterTiers", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    # création d'un en_cours
    data = {"auteur": loginDeux, "id_contenu": json.dumps(id), "provenance": "tiers", "version": 1}
    # création d'un contenu en_cours
    rv = client.post("/en_cours/newEnCours", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    id_en_cours = json.loads(rv.data)["_id"]

    supprimer_les_contenus(client, id, id_en_cours)
    rv = client.delete("/contenus/deleteTiers?id_tiers=" + json.dumps(id) + "&auteur=" + loginDeux + "&note=-",
                       headers=get_headers(client))
    assert rv.status_code == 200

    supprimerUtilisateur(client, login)
    supprimerUtilisateur(client, loginDeux)


def test_en_cours_directement_valide(client):
    # on récupère l'id du contenu en cours créé
    # Dans un premier temps, créer un contenu
    login = creerUtilisateurRandom(client)
    id = creerContenuRandom(client, login)


    data = {"auteur": login, "id_contenu": json.dumps(id), "provenance": "perso", "version": 1}
    # création d'un contenu en_cours
    rv = client.post("/en_cours/newEnCours", json=data, headers=get_headers(client))
    assert rv.status_code == 201

    id_en_cours = json.loads(rv.data)["_id"]

    # modification du texte du contenu en cours
    data = {"nouveau_texte": "Nouveau Texte du contenu", "nouveau_titre": "nouveauTitreContenu"}
    rv = client.patch("/contenus/changerTexte/" + json.dumps(id_en_cours), json=data, headers=get_headers(client))
    assert rv.status_code == 200

    data = {"auteur": "", "id_contenu_en_cours": json.dumps(id_en_cours)}
    rv = client.patch("/contenus/majContenu", json=data, headers=get_headers(client))
    assert rv.status_code == 200

    # test du texte sur le contenu d'id 'id'
    rv = client.get("/contenus/getTexteContenu/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200
    assert "Nouveau Texte du contenu" == json.loads(rv.data)


    # suppression du contenu "intial"
    rv = client.delete("/contenus/deletePerso/" + json.dumps(id), headers=get_headers(client))
    assert rv.status_code == 200

    supprimerUtilisateur(client, login)

